# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""Tests for the aggregated NIH+CXR14-PadChest dataset."""

import importlib

import pytest
from click.testing import CliRunner


@pytest.mark.parametrize(
    "name,padchest_name,combined_name",
    [
        ("default", "no_tb_idiap", "idiap"),
    ],
)
def test_split_consistency(name: str, padchest_name: str, combined_name: str):
    nih_cxr14 = importlib.import_module(
        f".{name}",
        "mednet.libs.classification.config.data.nih_cxr14",
    ).datamodule

    padchest = importlib.import_module(
        f".{padchest_name}",
        "mednet.libs.classification.config.data.padchest",
    ).datamodule

    combined = importlib.import_module(
        f".{combined_name}",
        "mednet.libs.classification.config.data.nih_cxr14_padchest",
    ).datamodule

    cxr14_loader = importlib.import_module(
        ".datamodule",
        "mednet.libs.classification.config.data.nih_cxr14",
    ).ClassificationRawDataLoader

    padchest_loader = importlib.import_module(
        ".datamodule",
        "mednet.libs.classification.config.data.padchest",
    ).ClassificationRawDataLoader

    for split in ("train", "validation", "test"):
        assert nih_cxr14.splits[split][0][0] == combined.splits[split][0][0]
        assert isinstance(nih_cxr14.splits[split][0][1], cxr14_loader)
        assert isinstance(combined.splits[split][0][1], cxr14_loader)

        if split != "test":
            # padchest has no test
            assert padchest.splits[split][0][0] == combined.splits[split][1][0]
            assert isinstance(padchest.splits[split][0][1], padchest_loader)
            assert isinstance(combined.splits[split][1][1], padchest_loader)


@pytest.mark.skip_if_rc_var_not_set("datadir.nih_cxr14")
@pytest.mark.skip_if_rc_var_not_set("datadir.padchest")
def test_database_check():
    from mednet.libs.classification.scripts.database import check

    runner = CliRunner()
    result = runner.invoke(check, ["--limit=10", "nih-cxr14-padchest"])
    assert (
        result.exit_code == 0
    ), f"Exit code {result.exit_code} != 0 -- Output:\n{result.output}"
